@@ -1,13 +1,19 @@
 """Support for NWS weather service."""
-from datetime import datetime, timedelta, timezone
-from statistics import mean
+from __future__ import annotations
 
+from datetime import datetime, timezone
+from statistics import mean
+from typing import Any, Dict, List, NamedTuple, Optional, Set, Tuple, Union
+
+from aiohttp import ClientSession
 from metar import Metar
 
-from .const import ALERT_ID, API_WEATHER_CODE
-from .nws import Nws
+from .const import ALERT_ID, API_WEATHER_CODE, Final
+from .forecast import DetailedForecast
+from .nws import Nws, NwsError
+from .units import convert_unit
 
-WIND_DIRECTIONS = [
+WIND_DIRECTIONS: Final = [
     "N",
     "NNE",
     "NE",
@@ -27,42 +33,25 @@ WIND_DIRECTIONS = [
 ]
 
 
-def unchanged(value):
-    """Return same value."""
-    return value
+WIND: Final = {name: idx * 360 / 16 for idx, name in enumerate(WIND_DIRECTIONS)}
 
 
-def f_to_c(fahrenheit):
-    """Convert to Celsius."""
-    return (fahrenheit - 32.0) / 1.8
+class MetarParam(NamedTuple):
+    """METAR conversion parameter"""
+
+    attr: str
+    units: Optional[str] = None
+    multiplier: Optional[float] = None
 
 
-def m_p_s_to_km_p_hr(m_p_s):
-    """Convert to km/hr."""
-    return m_p_s * 3.6
-
-
-unit_conversion = {
-    "unit:degC": unchanged,
-    "unit:degF": f_to_c,
-    "unit:km_h-1": unchanged,
-    "unit:m_s-1": m_p_s_to_km_p_hr,
-    "unit:m": unchanged,
-    "unit:Pa": unchanged,
-    "unit:percent": unchanged,
-    "unit:degree_(angle)": unchanged,
-}
-
-WIND = {name: idx * 360 / 16 for idx, name in enumerate(WIND_DIRECTIONS)}
-
-OBSERVATIONS = {
-    "temperature": ["temp", "C", None],
+OBSERVATIONS: Final[Dict[str, Optional[MetarParam]]] = {
+    "temperature": MetarParam("temp", "C"),
     "barometricPressure": None,
-    "seaLevelPressure": ["press", "HPA", 100],
+    "seaLevelPressure": MetarParam("press", "HPA", 100.0),
     "relativeHumidity": None,
-    "windSpeed": ["wind_speed", "MPS", 3.6],
-    "windDirection": ["wind_dir", None, None],
-    "visibility": ["vis", "M", None],
+    "windSpeed": MetarParam("wind_speed", "MPS", 3.6),
+    "windDirection": MetarParam("wind_dir"),
+    "visibility": MetarParam("vis", "M"),
     "elevation": None,
     "textDescription": None,
     "dewpoint": None,
@@ -79,13 +68,15 @@ OBSERVATIONS = {
     "heatIndex": None,
 }
 
+_WeatherCodes = List[Tuple[str, Optional[int]]]
 
-def convert_weather(weather):
+
+def convert_weather(weather: _WeatherCodes) -> _WeatherCodes:
     """Convert short code to readable name."""
     return [(API_WEATHER_CODE.get(w[0], w[0]), w[1]) for w in weather]
 
 
-def parse_icon(icon):
+def parse_icon(icon: str) -> Tuple[str, _WeatherCodes]:
     """
     Parse icon url to NWS weather codes.
 
@@ -100,7 +91,7 @@ def parse_icon(icon):
     weather = [i.split("?")[0] for i in icon_list[6:]]
     code = [w.split(",")[0] for w in weather]
     chance = [int(w.split(",")[1]) if len(w.split(",")) == 2 else None for w in weather]
-    return time, tuple(zip(code, chance))
+    return time, list(zip(code, chance))
 
 
 class SimpleNWS(Nws):
@@ -112,24 +103,32 @@ class SimpleNWS(Nws):
     By default, forecasts that end before now will be filtered out.
     """
 
-    def __init__(self, lat, lon, api_key, session, filter_forecast=True):
+    def __init__(
+        self: SimpleNWS,
+        lat: float,
+        lon: float,
+        api_key: str,
+        session: ClientSession,
+        filter_forecast: bool = True,
+    ):
         """Set up simplified NWS class."""
         super().__init__(session, api_key, (lat, lon))
 
         self.filter_forecast = filter_forecast
-        self._observation = None
-        self._metar_obs = None
-        self.station = None
-        self.stations = None
-        self._forecast = None
-        self._forecast_hourly = None
-        self._alerts_forecast_zone = []
-        self._alerts_county_zone = []
-        self._alerts_fire_weather_zone = []
-        self._alerts_all_zones = []
-        self._all_zones = []
+        self._observation: Optional[List[Dict[str, Any]]] = None
+        self._metar_obs: Optional[List[Metar.Metar]] = None
+        self.station: Optional[str] = None
+        self.stations: Optional[List[str]] = None
+        self._forecast: Optional[List[Dict[str, Any]]] = None
+        self._forecast_hourly: Optional[List[Dict[str, Any]]] = None
+        self._detailed_forecast: Optional[DetailedForecast] = None
+        self._alerts_forecast_zone: List[Dict[str, Any]] = []
+        self._alerts_county_zone: List[Dict[str, Any]] = []
+        self._alerts_fire_weather_zone: List[Dict[str, Any]] = []
+        self._alerts_all_zones: List[Dict[str, Any]] = []
+        self._all_zones: List[str] = []
 
-    async def set_station(self, station=None):
+    async def set_station(self: SimpleNWS, station: Optional[str] = None) -> None:
         """
         Set station or retrieve station list.
         If no station is supplied, the first retrieved value is set.
@@ -143,7 +142,7 @@ class SimpleNWS(Nws):
             self.station = self.stations[0]
 
     @staticmethod
-    def extract_metar(obs):
+    def extract_metar(obs: Dict[str, Any]) -> Optional[Metar.Metar]:
         """Return parsed metar if available."""
         metar_msg = obs.get("rawMessage")
         if metar_msg:
@@ -155,68 +154,77 @@ class SimpleNWS(Nws):
             metar_obs = None
         return metar_obs
 
-    async def update_observation(self, limit=0, start_time=None):
+    async def update_observation(
+        self: SimpleNWS, limit: int = 0, start_time: Optional[datetime] = None
+    ) -> None:
         """Update observation."""
         obs = await self.get_stations_observations(limit, start_time=start_time)
-        if obs is None:
-            return None
-        self._observation = obs
-        self._metar_obs = [self.extract_metar(iobs) for iobs in self._observation]
+        if obs:
+            self._observation = obs
+            self._metar_obs = [self.extract_metar(iobs) for iobs in self._observation]
 
-    async def update_forecast(self):
+    async def update_forecast(self: SimpleNWS) -> None:
         """Update forecast."""
         self._forecast = await self.get_gridpoints_forecast()
 
-    async def update_forecast_hourly(self):
+    async def update_forecast_hourly(self: SimpleNWS) -> None:
         """Update forecast hourly."""
         self._forecast_hourly = await self.get_gridpoints_forecast_hourly()
 
+    async def update_detailed_forecast(self: SimpleNWS) -> None:
+        """Update forecast."""
+        self._detailed_forecast = await self.get_detailed_forecast()
+
     @staticmethod
-    def _unique_alert_ids(alerts):
+    def _unique_alert_ids(alerts: List[Dict[str, Any]]) -> Set[str]:
         """Return set of unique alert_ids."""
         return {alert[ALERT_ID] for alert in alerts}
 
-    def _new_alerts(self, alerts, current_alerts):
+    def _new_alerts(
+        self: SimpleNWS,
+        alerts: List[Dict[str, Any]],
+        current_alerts: List[Dict[str, Any]],
+    ) -> List[Dict[str, Any]]:
         """Return new alerts."""
         current_alert_ids = self._unique_alert_ids(current_alerts)
         return [alert for alert in alerts if alert[ALERT_ID] not in current_alert_ids]
 
-    async def update_alerts_forecast_zone(self):
+    async def update_alerts_forecast_zone(self: SimpleNWS) -> List[Dict[str, Any]]:
         """Update alerts zone."""
         alerts = await self.get_alerts_forecast_zone()
         new_alerts = self._new_alerts(alerts, self._alerts_forecast_zone)
         self._alerts_forecast_zone = alerts
         return new_alerts
 
-    async def update_alerts_county_zone(self):
+    async def update_alerts_county_zone(self: SimpleNWS) -> List[Dict[str, Any]]:
         """Update alerts zone."""
         alerts = await self.get_alerts_county_zone()
         new_alerts = self._new_alerts(alerts, self._alerts_county_zone)
         self._alerts_county_zone = alerts
         return new_alerts
 
-    async def update_alerts_fire_weather_zone(self):
+    async def update_alerts_fire_weather_zone(self: SimpleNWS) -> List[Dict[str, Any]]:
         """Update alerts zone."""
         alerts = await self.get_alerts_fire_weather_zone()
         new_alerts = self._new_alerts(alerts, self._alerts_fire_weather_zone)
         self._alerts_fire_weather_zone = alerts
         return new_alerts
 
-    async def update_alerts_all_zones(self):
+    async def update_alerts_all_zones(self: SimpleNWS) -> List[Dict[str, Any]]:
         """Update all alerts zones."""
-        if not self.forecast_zone or not self.county_zone:
+        if not self.forecast_zone:
             await self.get_points()
+        if not self.forecast_zone:
+            raise NwsError("Forecast zone is missing")
         if not self._all_zones:
-            self._all_zones = {
-                self.forecast_zone,
-                self.county_zone,
-                self.fire_weather_zone,
-            }
+            # Use Set[str] to de-dupe zones
+            zones_set = {self.forecast_zone, self.county_zone, self.fire_weather_zone}
+            self._all_zones = [zone for zone in zones_set if zone]
         alerts_data = [
             await self.get_alerts_active_zone(zone) for zone in self._all_zones
         ]
 
-        alerts = []
+        alerts: List[Dict[str, Any]] = []
         for alert_list in alerts_data:
             for alert in alert_list:
                 if alert["id"] not in self._unique_alert_ids(alerts):
@@ -227,12 +235,14 @@ class SimpleNWS(Nws):
         return new_alerts
 
     @property
-    def all_zones(self):
+    def all_zones(self: SimpleNWS) -> List[str]:
         """All alert zones."""
         return self._all_zones
 
     @staticmethod
-    def extract_observation_value(observation, value):
+    def extract_observation_value(
+        observation: Dict[str, Any], value: str
+    ) -> Union[None, Tuple[float, str], str]:
         """Returns observation or observation value."""
         obs_value = observation.get(value)
         if obs_value is None:
@@ -245,34 +255,38 @@ class SimpleNWS(Nws):
         return observation[value]
 
     @property
-    def observation(self):
+    def observation(self: SimpleNWS) -> Optional[Dict[str, Any]]:
         """Observation dict"""
 
         if self._observation is None or self._observation == []:
             return None
 
-        data = {}
-        for obs, met in OBSERVATIONS.items():
+        data: Dict[str, Any] = {}
+        for obs, metar_param in OBSERVATIONS.items():
             obs_list = [
                 self.extract_observation_value(o, obs) for o in self._observation
             ]
             obs_item = next(iter([o for o in obs_list if o]), None)
             if isinstance(obs_item, tuple):
-                data[obs] = unit_conversion[obs_item[1]](obs_item[0])
+                data[obs] = convert_unit(obs_item[1], obs_item[0])
             else:
                 data[obs] = obs_item
 
-            if data[obs] is None and (
-                met is not None and self._metar_obs[0] is not None
+            if (
+                data[obs] is None
+                and metar_param is not None
+                and self._metar_obs is not None
+                and self._metar_obs[0] is not None
             ):
-                met_prop = getattr(self._metar_obs[0], met[0])
+                met_prop = getattr(self._metar_obs[0], metar_param.attr)
                 if met_prop:
-                    if met[1]:
-                        data[obs] = met_prop.value(units=met[1])
+                    if metar_param.units:
+                        data[obs] = met_prop.value(units=metar_param.units)
                     else:
                         data[obs] = met_prop.value()
-                    if met[2] is not None:
-                        data[obs] = data[obs] * met[2]
+                    if metar_param.multiplier is not None:
+                        data[obs] = data[obs] * metar_param.multiplier
+
         if data.get("icon"):
             time, weather = parse_icon(data["icon"])
             data["iconTime"] = time
@@ -283,34 +297,42 @@ class SimpleNWS(Nws):
         return data
 
     @property
-    def forecast(self):
+    def forecast(self: SimpleNWS) -> Optional[List[Dict[str, Any]]]:
         """Return forecast."""
         return self._convert_forecast(self._forecast, self.filter_forecast)
 
     @property
-    def forecast_hourly(self):
+    def forecast_hourly(self: SimpleNWS) -> Optional[List[Dict[str, Any]]]:
         """Return forecast hourly."""
         return self._convert_forecast(self._forecast_hourly, self.filter_forecast)
 
+    @property
+    def detailed_forecast(self: SimpleNWS) -> Optional[DetailedForecast]:
+        """Return detailed forecast.
+
+        Returns:
+            Optional[DetailedForecast]: Returns None if update_detailed_forecast
+            hasn't been called.
+        """
+        return self._detailed_forecast
+
     @staticmethod
-    def _convert_forecast(input_forecast, filter_forecast):
+    def _convert_forecast(
+        input_forecast: Optional[List[Dict[str, Any]]],
+        filter_forecast: bool,
+    ) -> List[Dict[str, Any]]:
         """Converts forecast to common dict."""
         if not input_forecast:
             return []
         forecast = []
+        now = datetime.now(timezone.utc)
         for forecast_entry in input_forecast:
             # get weather
             if filter_forecast:
                 end_time = forecast_entry.get("endTime")
                 if not end_time:
                     continue
-                # needed for python 3.6
-                # https://stackoverflow.com/questions/30999230/how-to-parse-timezone-with-colon
-                if end_time[-3] == ":":
-                    end_time = end_time[:-3] + end_time[-2:]
-                if datetime.now(timezone.utc) - datetime.strptime(
-                    end_time, "%Y-%m-%dT%H:%M:%S%z"
-                ) > timedelta(seconds=0):
+                if now > datetime.fromisoformat(end_time):
                     continue
             if forecast_entry.get("temperature"):
                 forecast_entry["temperature"] = int(forecast_entry["temperature"])
@@ -340,21 +362,21 @@ class SimpleNWS(Nws):
         return forecast
 
     @property
-    def alerts_forecast_zone(self):
+    def alerts_forecast_zone(self: SimpleNWS) -> List[Dict[str, Any]]:
         """Return alerts as a list of dict."""
         return self._alerts_forecast_zone
 
     @property
-    def alerts_county_zone(self):
+    def alerts_county_zone(self: SimpleNWS) -> List[Dict[str, Any]]:
         """Return alerts as a list of dict."""
         return self._alerts_county_zone
 
     @property
-    def alerts_fire_weather_zone(self):
+    def alerts_fire_weather_zone(self: SimpleNWS) -> List[Dict[str, Any]]:
         """Return alerts as a list of dict."""
         return self._alerts_fire_weather_zone
 
     @property
-    def alerts_all_zones(self):
+    def alerts_all_zones(self: SimpleNWS) -> List[Dict[str, Any]]:
         """Return alerts as a list of dict."""
         return self._alerts_all_zones
